﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data.Objects;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using Kendo.Mvc;
using Kendo.Mvc.UI;
using Data;
using Models;

namespace Business
{
    public class BusinessLayer
    {
        OTPTESTEntities _repository;

        public BusinessLayer(string connectionString)
        {
            _repository = new OTPTESTEntities(connectionString);
        }

        public void PopulateMaintenanceSportsViewData(ViewDataDictionary viewData, string selectedSportId)
        {
            viewData["hpas"] = (from h in this._repository.HPAs
                                orderby h.LastName, h.FirstName
                                select new SelectListItem
                                {
                                    Value = h.HPAID,
                                    Text = h.FirstName + " " + h.LastName
                                }).ToList();

            var nsos = (from o in this._repository.Organizations
                        where o.OrganizationTypeID == Constant.OrganizationTypeId.NSO
                        select new SelectListItem
                        {
                            Value = o.OrganizationID,
                            Text =  o.Name_EN
                        });

            viewData["nsos"] = nsos.OrderBy(o => o.Text).ToList();

            viewData["rankings"] = GetSportRankingsList();

            var sports = (from s in this._repository.Sports
                          where s.SportID == selectedSportId
                          && s.Active
                          select ": " +  s.Name_EN).SingleOrDefault();

            viewData["sportName"] = sports;

            viewData["genders"] = (from g in this._repository.Decodes
                                   where g.DecodeCategoryID == Constant.DecodeCategoryId.Gender
                                   orderby g.SortOrder
                                   select g.DecodeID_EN
                                   ).ToList();

            //viewData["yearId"] = "2013/2014";
            //viewData["selectedSports"] = "S_O_AC_C";

        }

        public void PopulateEvaluationsQuestionsViewData(ViewDataDictionary viewData, string yearId)
        {
            var id = GetFirstEvaluationVersionId(yearId);

            viewData["evaluationVersionId"] = id;

            viewData["versionId"] = (from v in this._repository.EvaluationVersions
                                     where v.EvaluationVersionID == id
                                     select v.VersionID).SingleOrDefault();

            string versionId = (string)viewData["versionId"];

            if (viewData["versionId"] == null)
            {
                viewData["versionId"] = 0;
            }
        }

        public void PopulateActionItemDecodeList(ViewDataDictionary viewData)
        {
            viewData["actionPriorities"] = GetDecodesByCategory(Constant.DecodeCategoryId.ActionItemPriority);
            viewData["actionStatus"] = GetDecodesByCategory(Constant.DecodeCategoryId.ActionItemStatus);
        }

        public List<MaintenanceSportsGridSportViewModel> GetMaintenanceSportsGridSportViewModel(string year, string selectedSport)
        {
            List<MaintenanceSportsGridSportViewModel> model = null;

            //IEnumerable<string> selectedSportIds = (from a in selectedSports select a.Id);

            model = (from s in this._repository.Sports
                     where s.SportLevelID == Constant.SportLevelId.Sport
                     && s.Active
                     && selectedSport.Contains(s.SportID)
                     let medalEvents = (from discipline in s.ChildSports
                                        where discipline.Active
                                        from sEvent in discipline.ChildSports
                                        where sEvent.Active
                                        && sEvent.MedalEvent != null
                                        && (bool)sEvent.MedalEvent
                                        select sEvent).Any()
                     select new MaintenanceSportsGridSportViewModel
                     {
                         SportId = s.SportID,
                         Name =  s.Name_EN,
                         Gender = s.Gender,
                         HPAID = s.HPAID,
                         HPAName = s.HPA == null ? "none" : s.HPA.FirstName + " " + s.HPA.LastName,
                         NSOID = s.OrganizationID,
                         NSOName = (from o in this._repository.Organizations
                                    where o.OrganizationID == s.OrganizationID
                                    select  o.Name_EN).FirstOrDefault(),
                         Benchmark = s.Benchmark,
                         HasComments = !string.IsNullOrEmpty(s.Comments),
                         SportCode = s.SportCode,
                         SportRankingCategoryID = (from r in s.SportRankings
                                                   where r.YearID == year
                                                   select r.SportRankingCategoryID == "NT" ? "" : r.SportRankingCategoryID).FirstOrDefault(),
                         Ranking = (from r in s.SportRankings
                                    where r.YearID == year
                                    select r.SportRankingCategoryID == "NT" ? "" : r.SportRankingCategoryID).FirstOrDefault(),
                         MedalEvent = medalEvents
                     }).OrderBy(s => s.SportCode).ToList();

            return model;
        }

        public List<MaintenanceSportsGridDisciplineViewModel> GetMaintenanceSportsGridDisciplineViewModel(string parentSportId)
        {
            List<MaintenanceSportsGridDisciplineViewModel> model = null;

            model = (from s in this._repository.Sports
                     where s.ParentSportID == parentSportId
                     && s.Active
                     select new MaintenanceSportsGridDisciplineViewModel
                     {
                         DisciplineId = s.SportID,
                         Name_EN = s.Name_EN,
                         Name_FR = s.Name_FR,
                         Benchmark = s.Benchmark,
                         HasComments = !string.IsNullOrEmpty(s.Comments),
                         SportCode = s.SportCode,
                         Gender = s.Gender,
                         MedalEvent = (from e in s.ChildSports
                                       where e.Active
                                     && e.MedalEvent != null
                                     && (bool)e.MedalEvent
                                       select e).Any()
                     }).ToList();

            return model;
        }

        public List<MaintenanceSportsGridEventViewModel> GetMaintenanceSportsGridEventViewModel(string parentSportId)
        {
            List<MaintenanceSportsGridEventViewModel> model = null;

            model = (from s in this._repository.Sports
                     where s.ParentSportID == parentSportId
                     && s.Active
                     select new MaintenanceSportsGridEventViewModel
                     {
                         EventId = s.SportID,
                         Name_EN = s.Name_EN,
                         Name_FR = s.Name_FR,
                         SportCode = s.SportCode,
                         Benchmark = s.Benchmark,
                         TeamSize = s.TeamSize,
                         HasComments = !string.IsNullOrEmpty(s.Comments),
                         MedalEvent = s.MedalEvent == null ? false : (bool)s.MedalEvent,
                         TeamSport = s.TeamSport == null ? false : (bool)s.TeamSport,
                         ResourceTracking = s.ResourceTracking == null ? false : (bool)s.ResourceTracking
                     }).ToList();

            return model;
        }

        public List<EvaluationsQuestionsEvaluationVersionGridViewModel> GetEvaluationsQuestionsEvaluationVersionGridViewModel(string yearId)
        {
            List<EvaluationsQuestionsEvaluationVersionGridViewModel> model = null;

            model = (from v in this._repository.EvaluationVersions
                     where v.YearID == yearId
                     select new EvaluationsQuestionsEvaluationVersionGridViewModel
                     {
                         EvaluationVersionId = v.EvaluationVersionID,
                         HasComments = !string.IsNullOrEmpty(v.Comments),
                         Name_EN = v.Name_EN,
                         Name_FR = v.Name_FR,
                         Name = v.Name_EN,
                         OrganizationTypeId = v.OrganizationTypeID,
                         OrganizationTypeName = v.OrganizationTypeID,
                         VersionId = v.VersionID,
                         VersionPhase = (from d in _repository.Decodes
                                         where d.DecodeID == v.VersionPhase && d.DecodeCategoryID == "VersionPhase"
                                         select d.DecodeID_EN).FirstOrDefault(),
                         YearId = v.YearID
                     }).ToList();

            return model;
        }

        public List<EvaluationsQuestionsEvaluationPillarsGridViewModel> GetEvaluationsQuestionsEvaluationPillarsGridViewModel(int evaluationVersionId, bool showDeleted)
        {
            List<EvaluationsQuestionsEvaluationPillarsGridViewModel> model = null;

            model = (from q in this._repository.EvaluationVersionQuestions
                     where q.EvaluationVersionID == evaluationVersionId
                     let p = q.EvaluationMasterQuestion.EvaluationMasterPillar
                     orderby p.SortOrder
                     select new EvaluationsQuestionsEvaluationPillarsGridViewModel
                     {
                         EvaluationMasterPillarId = p.EvaluationMasterPillarID,
                         EvaluationMasterPillarName = (from d in _repository.Decodes
                                                       where d.DecodeID == p.EvaluationMasterPillarID && d.DecodeCategoryID == "EvalMasterPillar"
                                                       select d.DecodeID_EN).FirstOrDefault(),
                         EvaluationVersionId = evaluationVersionId,
                         Description = p.Description_EN,
                         Name = p.Name_EN,
                         NumberOfQuestions = (from q2 in this._repository.EvaluationVersionQuestions
                                              where q2.EvaluationVersionID == evaluationVersionId
                                              && q2.EvaluationMasterQuestion.EvaluationMasterPillarID == p.EvaluationMasterPillarID
                                              && (showDeleted || (q2.QuestionPhase != Constant.QuestionPhase.DeletedPrevious && q2.QuestionPhase != Constant.QuestionPhase.DeletedCurrent))
                                              select q2).Count()

                     }).Distinct().ToList();

            model = model.OrderBy(x => x.EvaluationMasterPillarName).ToList();
            return model;
        }

        public List<EvaluationsQuestionsEvaluationPillarQuestionsGridViewModel> GetEvaluationsQuestionsEvaluationPillarQuestionsGridViewModel(int evaluationVersionId, string evaluationMasterPillarId, bool showDeleted)
        {
            List<EvaluationsQuestionsEvaluationPillarQuestionsGridViewModel> model = null;

            model = (from q in this._repository.EvaluationVersionQuestions
                     where q.EvaluationVersionID == evaluationVersionId
                     && q.EvaluationMasterQuestion.EvaluationMasterPillarID == evaluationMasterPillarId
                     && (showDeleted || (q.QuestionPhase != Constant.QuestionPhase.DeletedPrevious && q.QuestionPhase != Constant.QuestionPhase.DeletedCurrent))
                     orderby q.SortOrder
                     select new EvaluationsQuestionsEvaluationPillarQuestionsGridViewModel
                     {
                         EvaluationMasterQuestionId = q.EvaluationMasterQuestionID,
                         EvaluationMasterPillarId = evaluationMasterPillarId,
                         EvaluationVersionId = q.EvaluationVersionID,
                         EvaluationVersionQuestionId = q.EvaluationVersionQuestionID,
                         HasComments = !string.IsNullOrEmpty(q.Comments),
                         QuestionPhase = (from d in _repository.Decodes
                                          where d.DecodeID == q.QuestionPhase && d.DecodeCategoryID == "QuestionPhase"
                                          select d.DecodeID_EN).FirstOrDefault(),
                         SortOrder = q.SortOrder,
                         VersionQuestion = q.VersionQuestion,
                         SportId = q.SportID,
                         Editable = q.EvaluationVersion.VersionPhase == Constant.VersionPhase.Edit
                     }).ToList();

            //foreach (var question in model)
            //{
            //    question.SportName = GetFlattenedSportHeirachySportName(question.SportId);
            //}

            return model;
        }

        public Models.Evaluation GetEvaluation(int id)
        {
            Models.Evaluation evaluation = null;

            var evaluations = (from e in this._repository.Evaluations
                               where e.EvaluationID == id
                               select new Models.Evaluation
                               {
                                   Id = e.EvaluationID,
                                   VersionId = e.EvaluationVersionID,
                                   VersionYearId = e.EvaluationVersion.YearID,
                                   VersionName = e.EvaluationVersion.Name_EN,
                                   SportId = e.SportID,
                                   SportDesc = e.Sport.Name_EN,
                                   Phase = e.EvaluationPhase,
                                   Comment = e.Comments,
                                   VersionPhase = e.EvaluationVersion.VersionPhase
                               }).ToList();

            if (evaluations != null && evaluations.Count == 1)
            {
                evaluation = evaluations.First();
                evaluation = PopulateCodeDescription(evaluation);

                evaluation.Pillars = GetEvaluationQuestions(id);
            }

            return evaluation;
        }

        public List<Models.ActionItem> GetActionItems(string selectedStatusGrouping, string referenceId, string source, string type, int userId, IList<ListItem> selectedSports)
        {
            List<Models.ActionItem> actionItems = new List<Models.ActionItem>();

            List<Data.ActionItem> dataActionItems = new List<Data.ActionItem>();

            this._repository.ActionItems.MergeOption = System.Data.Objects.MergeOption.OverwriteChanges;

            if (string.IsNullOrEmpty(referenceId))
            {
                //links from nav menu, Funding Action Item item
                dataActionItems = (from a in this._repository.ActionItems
                                   join t in this._repository.ActionItemTypes on a.ActionItemTypeID equals t.ActionItemTypeID
                                   where t.SourceName_EN == source
                                   select a).ToList();
            }
            else if (!string.IsNullOrEmpty(type))
            {
                //links from three funding pages
                dataActionItems = (from a in this._repository.ActionItems
                                   where a.ActionItemTypeID == type && a.ReferenceID == referenceId
                                   select a).ToList();
            }
            else if (!string.IsNullOrEmpty(source))
            {
                //links from Funding Summary screen
                dataActionItems = (from a in this._repository.ActionItems
                                   join t in this._repository.ActionItemTypes on a.ActionItemTypeID equals t.ActionItemTypeID
                                   where t.SourceName_EN == source && a.ReferenceID == referenceId
                                   select a).ToList();
            }

            //apply status grouping
            dataActionItems = (from a in dataActionItems
                                where (selectedStatusGrouping == "O" && (a.ActionItemStatus == Constant.ActionItemStatus.New || a.ActionItemStatus == Constant.ActionItemStatus.InProgress))
                                    || (selectedStatusGrouping == "C" && !(a.ActionItemStatus == Constant.ActionItemStatus.New || a.ActionItemStatus == Constant.ActionItemStatus.InProgress))
                                orderby a.DueDate
                                select a).ToList();

            //apply filtered sports
            if (selectedSports != null)
            {
                IEnumerable<string> selectedSportIds = (from a in selectedSports select a.Id);
                dataActionItems = (from a in dataActionItems where selectedSportIds.Contains(a.SportID) select a).ToList();
            }

            actionItems = ConvertActionItems(dataActionItems);

            actionItems = PopulateCodeDescription(actionItems);

            return actionItems;
        }

        private string GetFilterString(string filterString, IList<Kendo.Mvc.IFilterDescriptor> filters)
        {
            foreach (var ifilter in filters)
            {
                if (ifilter.GetType() == typeof(Kendo.Mvc.CompositeFilterDescriptor))
                {
                    filterString = GetFilterString(filterString, ((Kendo.Mvc.CompositeFilterDescriptor)ifilter).FilterDescriptors);
                }
                else
                {
                    filterString = GetFilterString(filterString, ifilter);
                }
            }

            return filterString;
        }

        private string GetFilterString(string filterString, Kendo.Mvc.IFilterDescriptor iFilter)
        {
            var filter = (Kendo.Mvc.FilterDescriptor)iFilter;

            var filterValue = filter.Value.ToString().ToLower();

            var type = filter.Member.GetType();

            filterString += "Field:" + filter.Member + ","
                          + "Operator:" + filter.Operator.ToString() + ","
                          + "Type:" + type.ToString().Replace("System.", "") + ","
                          + "Content:" + filterValue + "; ";

            return filterString;
        }

        public List<ListItem> GetHumanGenders(bool includeNull)
        {
            List<ListItem> genders = (from g in this._repository.Decodes
                           where g.DecodeCategoryID == Constant.DecodeCategoryId.Gender && (g.DecodeID == Constant.Gender.Male || g.DecodeID == Constant.Gender.Female)
                           orderby g.SortOrder
                           select new ListItem
                           {
                               Id = g.DecodeID,
                               Description = g.DecodeID_EN
                           }).ToList();

            if (includeNull)
            {
                var nullItem = new ListItem { Id = null, Description = "{Not Set}" };
                genders.Insert(0, nullItem);
            }

            return genders;
        }

        public List<string> GetHumanGendersName(bool includeNull)
        {
            List<string> genders = (from g in this._repository.Decodes
                                    where g.DecodeCategoryID == "Gender" && (g.DecodeID == "M" || g.DecodeID == "F")
                                    orderby g.SortOrder
                                    select  g.DecodeID_EN
                                    ).ToList();

            if (includeNull)
            {
                genders.Insert(0, "{Not Set}");
            }

            return genders;
        }

        public string GetCurrentYearID()
        {
            // Year Id is separated by June.
            if (DateTime.Now.Month <= 6)
            {
                return DateTime.Now.AddYears(-1).Year.ToString() + "/" + DateTime.Now.AddYears(0).Year.ToString();
            }
            else
            {
                return DateTime.Now.AddYears(0).Year.ToString() + "/" + DateTime.Now.AddYears(1).Year.ToString();
            }
        }

        public List<SelectListItem> GetSportRankingsList()
        {
            List<SelectListItem> rankings = (from r in this._repository.SportRankingCategories
                                             select new SelectListItem
                                             {
                                                 Value = r.SportRankingCategoryID,
                                                 Text = r.OTPRank + r.OTPRankClass
                                             }).OrderBy(o => o.Text).ToList();

            rankings.Insert(0, new SelectListItem { Value = null, Text = "Select" });

            return rankings;
        }

        public IList<ListItem> GetSelectedSports(int sportLevelId)
        {
            var selectedSports = (from s in this._repository.Sports
                                  where s.SportLevelID == sportLevelId
                                  select new ListItem
                                  {
                                      Id = s.SportID,
                                      Description = s.Name_EN
                                  }).ToList();

            return selectedSports;
        }

        public int GetFirstEvaluationVersionId(string yearId)
        {
            return (from v in this._repository.EvaluationVersions
                    where v.YearID == yearId
                    orderby v.EvaluationVersionID
                    select v.EvaluationVersionID).FirstOrDefault();
        }

        public IEnumerable<Models.EvaluationPillar> GetEvaluationQuestions(int id)
        {
            //e.SportID is always at sport level
            //vq.SportID could be at sport level and up
            List<Models.EvaluationQuestion> questions = (from vq in this._repository.EvaluationVersionQuestions
                                                         join mq in this._repository.EvaluationMasterQuestions on vq.EvaluationMasterQuestionID equals mq.EvaluationMasterQuestionID
                                                         join e in this._repository.Evaluations on vq.EvaluationVersionID equals e.EvaluationVersionID
                                                         join s3 in this._repository.Sports on e.SportID equals s3.SportID
                                                         join s2 in this._repository.Sports on s3.ParentSportID equals s2.SportID
                                                         join s1 in this._repository.Sports on s2.ParentSportID equals s1.SportID
                                                         where e.EvaluationID == id && s3.SportLevelID == Constant.SportLevelId.Sport && s2.SportLevelID == Constant.SportLevelId.Season && s1.SportLevelID == Constant.SportLevelId.Olympic
                                                           && (e.SportID == vq.SportID || s3.ParentSportID == vq.SportID || s2.ParentSportID == vq.SportID || s1.ParentSportID == vq.SportID)
                                                           && (vq.QuestionPhase == Constant.QuestionPhase.New || vq.QuestionPhase == Constant.QuestionPhase.CopiedFromPrevious)
                                                         select new Models.EvaluationQuestion
                                                         {
                                                             VersionQuestionId = vq.EvaluationVersionQuestionID,
                                                             VersionId = vq.EvaluationVersionID,
                                                             MasterQuestionId = vq.EvaluationMasterQuestionID,
                                                             MasterPillarId = mq.EvaluationMasterPillarID,
                                                             SportId = vq.SportID,

                                                             Question = vq.VersionQuestion,
                                                             Phase = vq.QuestionPhase,
                                                             SortOrder = vq.SortOrder
                                                         }).ToList();

            questions = FillEvaluationAnswers(id, questions);
            questions.Sort(new Models.EvaluationQuestion.Comparer());

            //group questions by pillar
            List<Models.EvaluationPillar> pillars = (from g in
                                                         (from q in questions group q by q.MasterPillarId)
                                                     select new Models.EvaluationPillar { Id = g.Key, Questions = g.ToList() }).ToList();
            pillars = PopulateCodeDescription(pillars);
            pillars.Sort(new Models.EvaluationPillar.Comparer());

            return pillars;
        }

        private List<Models.EvaluationQuestion> FillEvaluationAnswers(int id, List<Models.EvaluationQuestion> questions)
        {
            var dataAnswers = (from q in this._repository.EvaluationQuestions
                               where q.EvaluationID == id
                               select q).ToList();

            foreach (var question in questions)
            {
                var answers = (from a in dataAnswers
                               where a.EvaluationVersionQuestionID == question.VersionQuestionId
                               select a).ToList();

                if (answers != null && answers.Count == 1)
                {
                    var answer = answers.First();

                    question.Id = answer.EvaluationQuestionID;

                    question.NotApplicable = answer.NotApplicable;
                    question.InitialStatus = answer.EvaluationStatusInitial;
                    question.CurrentStatus = answer.EvaluationStatusCurrent;
                    question.InitialAssessment = answer.HPAAssessmentInitial;
                    question.CurrentAssessment = answer.HPAAssessmentCurrent;
                    question.HasComments = !string.IsNullOrEmpty(answer.Comments);
                }
            }

            return questions;
        }

        public List<ListItem> GetCopyToYearsList(int evaluationVersionId)
        {
            List<ListItem> years = null;

            years = (from y in this._repository.Years
                     where (y.Active == null || (bool)y.Active)
                     orderby y.YearID
                     select new ListItem
                     {
                         Id = y.YearID,
                         Description = y.YearID
                     }).ToList();

            return years;
        }

        public IEnumerable<Models.Decode> GetDecodesByCategory(string categoryId)
        {
            var decodes = (from d in this._repository.Decodes
                           where d.DecodeCategoryID == categoryId
                           orderby d.SortOrder
                           select new Models.Decode
                            {
                                Id = d.DecodeID,
                                CategoryId = d.DecodeCategoryID,
                                Name = d.DecodeID_EN,
                                Comment = d.Comments,
                                SortOrder = d.SortOrder
                            }).ToList();

            return decodes;
        }

        private List<Models.ActionItem> ConvertActionItems(List<Data.ActionItem> dataActionItems)
        {
            return (from a in dataActionItems
                    select new Models.ActionItem
                    {
                        Id = a.ActionItemID,
                        TypeId = a.ActionItemTypeID,
                        ReferenceId = a.ReferenceID,
                        OrganizationId = a.OrganizationID,
                        SportId = a.SportID,
                        Description = a.Description,
                        DueDate = a.DueDate,
                        Status = a.ActionItemStatus,
                        Priority = a.ActionItemPriority,
                        Comment = a.Comments,
                        HasComment = !string.IsNullOrEmpty(a.Comments),
                        UserId = 9999,
                        ControllerName = GetControllerName(a.ActionItemTypeID),
                        ActionName = GetActionName(a.ActionItemTypeID)
                    }).ToList();
        }

        private List<Models.ActionItem> PopulateCodeDescription(List<Models.ActionItem> actionItems)
        {
            if (actionItems == null) return null;

            IEnumerable<Models.Decode> actionItemTypes = GetActionItemTypes();
            IEnumerable<Models.Decode> statusTypes = GetDecodesByCategory(Constant.DecodeCategoryId.ActionItemStatus);
            IEnumerable<Models.Decode> priorityTypes = GetDecodesByCategory(Constant.DecodeCategoryId.ActionItemPriority);
            IEnumerable<Models.Organization> orgs = GetAllOrganizations();
            IEnumerable<ListItem> sports = GetAllSportsOnSportLevel();

            foreach (var item in actionItems)
            {
                item.TypeDesc = (from t in actionItemTypes where t.Id == item.TypeId select t.Name).SingleOrDefault();
                item.StatusDesc = (from t in statusTypes where t.Id == item.Status select t.Name).SingleOrDefault();
                item.PriorityDesc = (from t in priorityTypes where t.Id == item.Priority select t.Name).SingleOrDefault();

                item.OrganizationDesc = (from o in orgs where o.Id == item.OrganizationId select o.Name).SingleOrDefault();
                item.SportDesc = (from s in sports where s.Id == item.SportId select s.Description).SingleOrDefault();
            }

            return actionItems;
        }

        private Models.Evaluation PopulateCodeDescription(Models.Evaluation evaluation)
        {
            if (evaluation == null) return null;

            var types = GetDecodesByCategory(Constant.DecodeCategoryId.EvaluationPhase);

            evaluation.PhaseDesc = (from d in types where d.Id == evaluation.Phase select d.Name).FirstOrDefault();

            return evaluation;
        }

        private List<Models.EvaluationPillar> PopulateCodeDescription(List<Models.EvaluationPillar> pillars)
        {
            if (pillars == null) return pillars;

            var dataPillars = (from p in this._repository.EvaluationMasterPillars
                               select p).ToList();

            foreach (var pillar in pillars)
            {
                var dataPillar = (from p in dataPillars where p.EvaluationMasterPillarID == pillar.Id select p).SingleOrDefault();

                if (dataPillar == null) continue;

                pillar.Id_Name = (from d in this._repository.Decodes
                                  where d.DecodeCategoryID == "EvalMasterPillar" && d.DecodeID == pillar.Id
                                  select d.DecodeID_EN).SingleOrDefault();
                pillar.Name = dataPillar.Name_EN;
                pillar.Description = dataPillar.Description_EN;
                pillar.SortOrder = dataPillar.SortOrder;
                pillar.Comment = dataPillar.Comments;
            }

            return pillars;
        }

        public int GetSelectedTabIndex(Models.Evaluation evaluation, string selectedTabName)
        {
            int selectedTabIndex = 0;

            if (evaluation == null || evaluation.Pillars == null) return selectedTabIndex;

            selectedTabIndex = (from p in evaluation.Pillars select p.Id).ToList().IndexOf(selectedTabName);

            if (selectedTabIndex < 0 || selectedTabIndex >= evaluation.Pillars.Count()) selectedTabIndex = 0;

            return selectedTabIndex;
        }

        public IEnumerable<Models.Decode> GetActionItemTypes()
        {
            var decodes = (from a in this._repository.ActionItemTypes
                            select new Models.Decode
                            {
                                Id = a.ActionItemTypeID,
                                Name = a.CategoryName_EN,
                                Comment = a.Comments,
                                CategoryId = a.SourceName_EN,
                                SortOrder = 0
                            }).ToList();

            return decodes;
        }

        public List<Models.Organization> GetAllOrganizations()
        {
            var organizations = (from o in this._repository.Organizations
                                    select new Models.Organization
                                    {
                                        Id = o.OrganizationID,
                                        Type = o.OrganizationTypeID,
                                        Name = o.Name_EN
                                    }).ToList();

            return organizations;
        }

        public List<ListItem> GetAllSportsOnSportLevel()
        {
            List<ListItem> sports = null;

            sports = (from s in this._repository.Sports
                      where s.Active
                      && s.SportLevelID == Constant.SportLevelId.Sport
                      select new ListItem
                      {
                          Id = s.SportID,
                          Description = s.Name_EN
                      }).OrderBy(s => s.Description).ToList();

            return sports;
        }

        private string GetControllerName(string actionItemType)
        {
            if (actionItemType == Constant.ActionItemType.FundingRequest
                || actionItemType == Constant.ActionItemType.FundingRecommendation
                || actionItemType == Constant.ActionItemType.FundingRevision)
            {
                return "Funding";
            }

            return "Home";
        }

        private string GetActionName(string actionItemType)
        {
            if (actionItemType == Constant.ActionItemType.FundingRequest)
                return Constant.FundingAction.Request;

            if (actionItemType == Constant.ActionItemType.FundingRecommendation)
                return Constant.FundingAction.Recommend;

            if (actionItemType == Constant.ActionItemType.FundingRevision)
                return Constant.FundingAction.Revise;

            return "EvaluationForm";
        }

    }
}
