﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Configuration;
using System.Web;
using System.Web.Mvc;
using Kendo.Mvc;
using Kendo.Mvc.UI;
using Kendo.Mvc.Extensions;
using Business;
using Models;

namespace MVC_Controls_Kendo.Controllers
{
    public class HomeController : Controller
    {
        BusinessLayer _business;
        string yearId = "2013/2014";
        string sportId = Constant.SportId.All;

        public HomeController()
        {
            _business = new BusinessLayer(ConfigurationManager.ConnectionStrings["OTPTESTEntities"].ConnectionString);
        }

        #region Sport Page Info

        public ActionResult SportPage()
        {
            this._business.PopulateMaintenanceSportsViewData(ViewData, sportId);

            return View();
        }

        public ActionResult MaintenanceSportsAjax(DataSourceRequest request)
        {
            //retreive sports
            var model = this._business.GetMaintenanceSportsGridSportViewModel(yearId, "S_O_AC_C");

            DataSourceResult sports = model.ToDataSourceResult(request);

            return Json(sports, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateMaintenanceSports(DataSourceRequest request, string sportId)
        {
            var updatedModel = new MaintenanceSportsGridSportViewModel();

            return MaintenanceSportsAjax(request);
        }

        public ActionResult MaintenanceDisciplinesAjax(DataSourceRequest request, string sportId)
        {
            var model = this._business.GetMaintenanceSportsGridDisciplineViewModel(sportId);

            DataSourceResult disciplines = model.ToDataSourceResult(request);

            return Json(disciplines, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InsertMaintenanceDisciplines(DataSourceRequest request, string parentSportId)
        {
            var updatedModel = new MaintenanceSportsGridDisciplineViewModel();

            return MaintenanceDisciplinesAjax(request, parentSportId);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateMaintenanceDisciplines(DataSourceRequest request, string id)
        {
            var updatedModel = new MaintenanceSportsGridDisciplineViewModel();
            updatedModel.DisciplineId = id;

            return MaintenanceDisciplinesAjax(request, id);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeleteMaintenanceDisciplines(DataSourceRequest request, string id)
        {
            return MaintenanceDisciplinesAjax(request, id);
        }

        public ActionResult MaintenanceEventsAjax(DataSourceRequest request, string disciplineId)
        {
            var model = this._business.GetMaintenanceSportsGridEventViewModel(disciplineId);

            DataSourceResult events = model.ToDataSourceResult(request);

            return Json(events, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InsertMaintenanceEvents(DataSourceRequest request, string parentDisciplineId)
        {
            var updatedModel = new MaintenanceSportsGridEventViewModel();

            return MaintenanceEventsAjax(request, parentDisciplineId);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateMaintenanceEvents(DataSourceRequest request, string id)
        {
            var updatedModel = new MaintenanceSportsGridEventViewModel();
            updatedModel.EventId = id;

            return MaintenanceEventsAjax(request, id);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeleteMaintenanceEvents(DataSourceRequest request, string id)
        {
            return MaintenanceEventsAjax(request, id);
        }

        #endregion Sport Page Info

        #region Evaluation Questions

        public ActionResult EvaluationQuestions()
        {
            this._business.PopulateEvaluationsQuestionsViewData(ViewData, yearId);

            return View();
        }

        public ActionResult LoadEvaluationsQuestionsEvaluationVersionGridAjax(DataSourceRequest request)
        {
            var model = this._business.GetEvaluationsQuestionsEvaluationVersionGridViewModel(yearId);

            if (model == null)
                return null;

            DataSourceResult result = model.ToDataSourceResult(request);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InsertEvaluationVersion(DataSourceRequest request)
        {
            var updatedModel = new EvaluationsQuestionsEvaluationVersionGridViewModel();
            updatedModel.YearId = yearId;

            return LoadEvaluationsQuestionsEvaluationVersionGridAjax(request);
        }
        
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateEvaluationVersion(DataSourceRequest request, string id)
        {
            var updatedModel = new EvaluationsQuestionsEvaluationVersionGridViewModel();
            updatedModel.EvaluationVersionId = int.Parse(id);
            updatedModel.YearId = yearId;

            return LoadEvaluationsQuestionsEvaluationVersionGridAjax(request);
        }

        public ActionResult LoadEvaluationsQuestionsEvaluationPillarsGridAjax(DataSourceRequest request, string evaluationVersionId, bool showDeleted)
        {
            int id = 0;

            if (string.IsNullOrEmpty(evaluationVersionId))
            {
                id = this._business.GetFirstEvaluationVersionId(yearId);
            }
            else
            {
                id = int.Parse(evaluationVersionId);
            }

            var model = this._business.GetEvaluationsQuestionsEvaluationPillarsGridViewModel(id, showDeleted);

            if (model == null)
                return null;

            DataSourceResult result = model.ToDataSourceResult(request);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        public ActionResult LoadEvaluationsQuestionsEvaluationPillarQuestionsGridAjax(DataSourceRequest request, string evaluationVersionId, string evaluationMasterPillarId, bool showDeleted)
        {
            var model = this._business.GetEvaluationsQuestionsEvaluationPillarQuestionsGridViewModel(int.Parse(evaluationVersionId), evaluationMasterPillarId, showDeleted);

            if (model == null)
                return null;

            DataSourceResult result = model.ToDataSourceResult(request);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InsertEvaluationVersionQuestion(DataSourceRequest request, string prevEvaluationVersionId, string prevEvaluationMasterPillarId)
        {
            var updatedModel = new EvaluationsQuestionsEvaluationPillarQuestionsGridViewModel();
            updatedModel.EvaluationVersionId = int.Parse(prevEvaluationVersionId);
            updatedModel.EvaluationMasterPillarId = prevEvaluationMasterPillarId;

            return LoadEvaluationsQuestionsEvaluationPillarQuestionsGridAjax(request, prevEvaluationVersionId, prevEvaluationMasterPillarId, false);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateEvaluationVersionQuestion(DataSourceRequest request, string id, string evaluationVersionId, string evaluationMasterPillarId)
        {
            var updatedModel = new EvaluationsQuestionsEvaluationPillarQuestionsGridViewModel();
            updatedModel.EvaluationVersionQuestionId = int.Parse(id);

            return LoadEvaluationsQuestionsEvaluationPillarQuestionsGridAjax(request, evaluationVersionId, evaluationMasterPillarId, false);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeleteEvaluationVersionQuestion(DataSourceRequest request, string id, string evaluationVersionId, string evaluationMasterPillarId)
        {
            //this._business.DeleteEvaluationVersionQuestion(int.Parse(id));

            return LoadEvaluationsQuestionsEvaluationPillarQuestionsGridAjax(request, evaluationVersionId, evaluationMasterPillarId, false);
        }

        public ActionResult CopyEvaluationVersion(string copyEvaluationVersionId, string copyToYear)
        {
            //this._business.CopyEvaluationVersion(int.Parse(copyEvaluationVersionId), copyToYear);
            return RedirectToAction("Questions", "Evaluations");
        }

        public JsonResult EvaluationCopyToYearAjax(string evaluationVersionId)
        {
            var years = this._business.GetCopyToYearsList(int.Parse(evaluationVersionId));

            return Json(years, JsonRequestBehavior.AllowGet);
        }

        #endregion Evaluation Questions

        #region Action Items

        public ActionResult ActionItems()
        {
            //string currentPageId = Constant.PageId.EvaluationAction;

            //bool validUserOnPage = this.ValidateUserOnPage(currentPageId);
            //if (!validUserOnPage)
            //{
            //    return this.RedirectToUserHomePage();
            //}

            //var model = new EvaluationActionItemViewModel();
            //model.Menu = this.Menu(currentPageId);
            //model.CurrentLanguageCode = this.CurrentLanguageCode();
            //model.CurrentUserName = this.CurrentUserName();

            //model.Editable = (model.Menu.Menu.WriteAccess != Constant.AccessLevel.No);

            //model.SelectorItems = new List<Models.ListItem>() { new Models.ListItem { Id = "O", Description = Shared.SharedStrings.ListItem_Open }, new Models.ListItem { Id = "C", Description = Shared.SharedStrings.Listitem_Close } };
            //model.SelectorValue = this.SelectedActionItemStatus();

            this._business.PopulateActionItemDecodeList(ViewData);

            return View();
        }

        public ActionResult GetActionItemsAjax(DataSourceRequest request, string referenceId, string type)
        {
            var selectedSports = this._business.GetSelectedSports(Constant.SportLevelId.Sport);

            var model = this._business.GetActionItems("O", referenceId, Constant.ActionItemSource.Evaluation, type, 60, selectedSports);

            if (model == null)
                return null;

            DataSourceResult result = model.ToDataSourceResult(request);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateActionItemsAjax(DataSourceRequest request, string referenceId, string type)
        {
            ActionItem actionItem = new ActionItem();
            //if (TryUpdateModel(actionItem))
            //{
            //    var result = this._actionItemBL.UpdateActionItem(actionItem);
            //}

            return GetActionItemsAjax(request, referenceId, type);
        }

        #endregion Action Items

        #region EvaluationForm

        public ActionResult EvaluationForm(int? id, string spillar)
        {
            string currentPageId = Constant.PageId.EvaluationForm;

            var model = new EvaluationFormViewModel();
            //model.Menu = this.Menu(currentPageId);
            //model.CurrentLanguageCode = this.CurrentLanguageCode();
            //model.CurrentUserName = this.CurrentUserName();

            if (id.HasValue)
            {
                var evaluation = this._business.GetEvaluation(id.Value);

                if (evaluation != null)
                {
                    model.Evaluation = evaluation;

                    model.SelectedTabIndex = this._business.GetSelectedTabIndex(model.Evaluation, spillar);

                    //model.ToolbarButtons = GetToolbarButtons(model.Menu.Menu.PageMenuItems);
                }
            }

            ViewData["actionPriorities"] = this._business.GetDecodesByCategory(Constant.DecodeCategoryId.ActionItemPriority);
            ViewData["actionStatus"] = this._business.GetDecodesByCategory(Constant.DecodeCategoryId.ActionItemStatus);

            //if (TempData.ContainsKey("Errors"))
            //{
            //    ModelState.AddModelError("", TempData["Errors"].ToString());
            //}

            return View(model);
        }

        //public ActionResult EvaluationFormPillar(int id, string pillarId, bool hasActionItem)
        //{
        //    var model = new EvaluationPillarViewModel();

        //    Evaluation evaluation = this._business.GetEvaluation(id);

        //    if (evaluation != null && evaluation.Pillars != null)
        //    {
        //        model.EvaluationId = evaluation.Id;
        //        model.SportId = evaluation.SportId;

        //        model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

        //        model.HasActionItem = hasActionItem;
        //        model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
        //        model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
        //        model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;

        //        model.ActionItemRatio = this._business.GetActionItemRatioForEvaluationPillar(id, pillarId);
        //    }

        //    return PartialView(model);
        //}

        #region EvaluationFormTab

        public ActionResult EvaluationFormTab_Overall(int id)
        {
            var model = new EvaluationPillarViewModel();
            string pillarId = "Overall";

            Evaluation evaluation = this._business.GetEvaluation(id);

            if (evaluation != null && evaluation.Pillars != null)
            {
                model.EvaluationId = evaluation.Id;
                model.SportId = evaluation.SportId;

                model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

                //model.Editable = editable;
                //model.HasActionItem = hasActionItem;
                model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
                model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
                model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;

                //model.ActionItemRatio = this._business.GetActionItemRatioForEvaluationPillar(id, pillarId);
            }

            return PartialView(model);
        }

        public ActionResult EvaluationFormTab_Coaching(int id)
        {
            var model = new EvaluationPillarViewModel();
            string pillarId = "Coaching";

            Evaluation evaluation = this._business.GetEvaluation(id);

            if (evaluation != null && evaluation.Pillars != null)
            {
                model.EvaluationId = evaluation.Id;
                model.SportId = evaluation.SportId;

                model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

                model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
                model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
                model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;
            }

            return PartialView(model);
        }

        public ActionResult EvaluationFormTab_Athlete(int id)
        {
            var model = new EvaluationPillarViewModel();
            string pillarId = "Athlete";

            Evaluation evaluation = this._business.GetEvaluation(id);

            if (evaluation != null && evaluation.Pillars != null)
            {
                model.EvaluationId = evaluation.Id;
                model.SportId = evaluation.SportId;

                model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

                model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
                model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
                model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;
            }

            return PartialView(model);
        }

        public ActionResult EvaluationFormTab_TandC(int id)
        {
            var model = new EvaluationPillarViewModel();
            string pillarId = "TandC";

            Evaluation evaluation = this._business.GetEvaluation(id);

            if (evaluation != null && evaluation.Pillars != null)
            {
                model.EvaluationId = evaluation.Id;
                model.SportId = evaluation.SportId;

                model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

                model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
                model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
                model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;
            }

            return PartialView(model);
        }

        public ActionResult EvaluationFormTab_IST(int id)
        {
            var model = new EvaluationPillarViewModel();
            string pillarId = "IST";

            Evaluation evaluation = this._business.GetEvaluation(id);

            if (evaluation != null && evaluation.Pillars != null)
            {
                model.EvaluationId = evaluation.Id;
                model.SportId = evaluation.SportId;

                model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

                model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
                model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
                model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;
            }

            return PartialView(model);
        }

        public ActionResult EvaluationFormTab_Benchmarks(int id)
        {
            var model = new EvaluationPillarViewModel();
            string pillarId = "Benchmarks";

            Evaluation evaluation = this._business.GetEvaluation(id);

            if (evaluation != null && evaluation.Pillars != null)
            {
                model.EvaluationId = evaluation.Id;
                model.SportId = evaluation.SportId;

                model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

                model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
                model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
                model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;
            }

            return PartialView(model);
        }

        public ActionResult EvaluationFormTab_Governance(int id)
        {
            var model = new EvaluationPillarViewModel();
            string pillarId = "Governance";

            Evaluation evaluation = this._business.GetEvaluation(id);

            if (evaluation != null && evaluation.Pillars != null)
            {
                model.EvaluationId = evaluation.Id;
                model.SportId = evaluation.SportId;

                model.Pillar = (from p in evaluation.Pillars where p.Id == pillarId select p).FirstOrDefault();

                model.InitialStatusEditable = evaluation.Phase == Constant.EvaluationPhase.Edit;
                model.CurrentStatusVisible = evaluation.Phase != Constant.EvaluationPhase.Edit;
                model.CanEditSportQuestions = model.Editable && evaluation.VersionPhase != Constant.VersionPhase.Closed;
            }

            return PartialView(model);
        }

        public ActionResult EvaluationFormTab_Ajax(DataSourceRequest request, int id, string pilId)
        {
            IEnumerable<EvaluationPillar> pillars = this._business.GetEvaluationQuestions(id);

            var pillar = (from p in pillars where p.Id == pilId select p).FirstOrDefault();

            if (pillar.Questions == null)
                return null;

            List<EvaluationQuestion> questions = (List<EvaluationQuestion>)pillar.Questions;

            DataSourceResult result = questions.ToDataSourceResult(request);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult InsertEvaluationFormTab_Ajax(DataSourceRequest request, int preEvaluationId, string prePillarId, string preEvaluationSportId)
        {
            var postedModel = new EvaluationQuestion();

            if (TryUpdateModel(postedModel))
            {
                postedModel.Id = 0;

                //SavePillarQuestions(preEvaluationId, prePillarId, preEvaluationSportId, postedModel);
            }

            return EvaluationFormTab_Ajax(request, preEvaluationId, prePillarId);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult UpdateEvaluationFormTab_Ajax(DataSourceRequest request, int evaluationId, string pillarId, string evaluationSportId)
        {
            var postedModel = new EvaluationQuestion();

            if (TryUpdateModel(postedModel))
            {
                //SavePillarQuestions(evaluationId, pillarId, evaluationSportId, postedModel);
            }

            return EvaluationFormTab_Ajax(request, evaluationId, pillarId);
        }

        #endregion EvaluationFormTab

        #endregion EvaluationForm

        public ActionResult Index()
        {
            ViewBag.Message = "OTP Test Web Pages";

            return View();
        }

        public ActionResult About()
        {
            ViewBag.Message = "Your app description page.";

            return View();
        }

        public ActionResult Contact()
        {
            ViewBag.Message = "Your contact page.";

            return View();
        }
    }
}
